#include "fileBlock.h"
#include "globalVariable.h"
#include "md5.h"

using namespace std;

fileBlock::fileBlock()
{
	
}

void fileBlock::getFileMd5(string filename)
{
	FILE* fp;

	const int BUFSIZE = 128*128;
	char cbuf2[BUFSIZE];
	for(int i=0; i<BUFSIZE; i++)
		cbuf2[i] = 0;

	int numread;
	int filesize = 0;

	const char *fileName = filename.c_str();

	md5_state_t state;
	md5_byte_t digest[16];
	char hex_output[16*2 + 1];
	int di;

	md5_init(&state);

	fp = fopen(fileName, "rb");
	if(fp==NULL)
	{
		cout<<"Failed to open file!"<<endl;
		errornum++;
		return;
	}

	while(1)
	{
		for(int i=0; i<BUFSIZE; i++)
			cbuf2[i] = 0;

		numread = fread(cbuf2, 1, BUFSIZE, fp);
		filesize += numread;

		if (!numread)
			break;

		md5_append(&state, (const md5_byte_t *)cbuf2, numread);
	}

	//cout<<dec<<filesize<<endl;
	FileSize = filesize;

	md5_finish(&state, digest);
	for (di = 0; di < 16; ++di)
	    sprintf(hex_output + di * 2, "%02x", digest[di]);
	//cout<<hex_output<<endl;
	strcpy(FileMd5, hex_output);

	FileBlock(fp, filesize);
	fclose(fp);

	strcat(fileBasicInfo, FileMd5);
	strcat(fileBasicInfo, "\t");
}

void fileBlock::FileBlock(FILE* fp, int filesize)
{
	fileMd5List = "";

	if(filesize > 128*1024)
		getBlockMd5(fp, 128);
	else
	{
		fileMd5List += FileMd5;
		fileMd5List += "\t";
	}

	if(filesize > 256*1024)
		getBlockMd5(fp, 256);
	else
	{
		fileMd5List += FileMd5;
		fileMd5List += "\t";
	}

	if(filesize > 512*1024)
		getBlockMd5(fp, 512);
	else
	{
		fileMd5List += FileMd5;
		fileMd5List += "\t";
	}

	if(filesize > 1024*1024)
		getBlockMd5(fp, 1024);
	else
	{
		fileMd5List += FileMd5;
		fileMd5List += "\t";
	}

	if(filesize > 2048*1024)
		getBlockMd5(fp, 2048);
	else
	{
		fileMd5List += FileMd5;
		fileMd5List += "\t";
	}

	if(filesize > 4096*1024)
		getBlockMd5(fp, 4096);
	else
	{
		fileMd5List += FileMd5;
		fileMd5List += "\t";
	}

	if(filesize > 8192*1024)
		getBlockMd5(fp, 8192);
	else
	{
		fileMd5List += FileMd5;
		fileMd5List += "\t";
	}

	if(filesize > 16384*1024)
		getBlockMd5(fp, 16384);
	else
	{
		fileMd5List += FileMd5;
		fileMd5List += "\n\n";
	}
}

void fileBlock::getBlockMd5(FILE* fp, int blocksize)
{
	const int BUFSIZE = 1024;
	char cbuf2[BUFSIZE];
	for(int i=0; i<BUFSIZE; i++)
		cbuf2[i] = 0;

	int numread;
	int blocknum;
	int num = 0;

	md5_state_t state;
	md5_byte_t digest[16];
	char hex_output[16*2 + 1];
	int di;
	md5_init(&state);
	
	fseek(fp, 0, SEEK_SET);
	while(1)
	{
		for(int i=0; i<BUFSIZE; i++)
			cbuf2[i] = 0;

		numread = fread(cbuf2, 1, BUFSIZE, fp);

		if (!numread)
			break;
		
		if(num < blocksize)
		{
			md5_append(&state, (const md5_byte_t *)cbuf2, numread);
			num++;
		}
		else
		{
			md5_finish(&state, digest);
			for (di = 0; di < 16; ++di)
				sprintf(hex_output + di * 2, "%02x", digest[di]);
			fileMd5List += hex_output;
			fileMd5List += "-";

			md5_init(&state);
			md5_append(&state, (const md5_byte_t *)cbuf2, numread);
			num = 1;
		}
	}

	if(num!=0)
	{
		md5_finish(&state, digest);
		for (di = 0; di < 16; ++di)
			sprintf(hex_output + di * 2, "%02x", digest[di]);
		fileMd5List += hex_output;
	}
	
	if(blocksize != 16384)
		fileMd5List += "\t";
	else
		fileMd5List += "\n\n";
}

